<?php

function nodestream_theme($existing, $type, $theme, $path) {
  return array(
    'nodestream_ns_feature' => array(
      'variables' => array('element' => NULL),
    ),
  );
}

function theme_nodestream_ns_feature($variables) {
  var_dump($variables);
  return '';
}

function nodestream_install_tasks() {
  return array(
    'nodestream_pick_features_form' => array(
      'display_name' => st('Configure NodeStream'),
      'type' => 'form',
    ),
    'nodestream_install_features' => array(
      'display_name' => st('Install NodeStream features'),
      'type' => 'batch',
    ),
    'nodestream_finish' => array(
      'display_name' => st('Final configuration'),
      'display' => FALSE,
      'type' => 'normal',
    ),
  );
}

/**
 * Feature selection form. The user get's to select the feature she wants
 * to have on the site during installation.
 */
function nodestream_pick_features_form($form, &$form_state, &$install_state) {
  $features = array();
  drupal_set_title(st('Configure NodeStream'));
  $modules = $form_state['modules'] = system_rebuild_module_data();

  foreach ($modules as $module_name => $module) {
    if (isset($module->info['nodestream_type'])) {
      $features[$module->info['nodestream_type']][$module_name] = $module;
    }
  }
  $form['welcome']['#markup'] = '<h1 class="title">Select features</h1><p>' . st('Welcome to NodeStream! NodeStream comes with a wide array
    of features and depending on which you choose, you can create different kinds of sites. The features are
    presented below.') . '</p>';
  $form['features'] = array(
    '#tree' => TRUE,
  );
  $types = module_invoke_all('nodestream_feature_types');
  foreach ($types as $type_name => $type) {
    if (isset($features[$type_name])) {
      $form['features'][$type_name] = array(
        '#type' => 'fieldset',
        '#title' => $type['title'],
        '#collapsible' => TRUE,
        '#collapsed' => isset($type['collapsed']) ? $type['collapsed'] : TRUE,
        '#description' => isset($type['description']) ? $type['description'] : '',
      );
      foreach ($features[$type_name] as $name => $feature) {
        $form['features'][$type_name][$name] = array(
          '#type' => 'fieldset',
          '#title' => $feature->info['name'],
          '#description' => $feature->info['description'],
        );
        if (isset($feature->info['nodestream_screenshot'])) {
          $form['features'][$type_name][$name]['screenshot']['#markup'] = theme('image', array(
              'path' => drupal_get_path('module', $name) . '/' . $feature->info['nodestream_screenshot'],
              'width' => 500,
            )
          );
        }
        $form['features'][$type_name][$name]['enabled'] = array(
          '#type' => 'checkbox',
          '#title' => st('Enable this feature'),
        );
      }
    }
  }
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => st('Save and continue'),
    '#weight' => 15,
  );
  return $form;
}

/**
 * Submit handler for feature picking form.
 */
function nodestream_pick_features_form_submit($form, &$form_state) {
  variable_set('nodestream_enable_features', $form_state['values']['features']);
}

/**
 * Install selected features from the previous section.
 */
function nodestream_install_features(&$install_state) {
  // Get all selected features out of the array.
  $feature_groups = variable_get('nodestream_enable_features', array());
  $modules = array();
  $files = system_rebuild_module_data();
  variable_del('nodestream_enable_features');
  foreach ($feature_groups as $group => $features) {
   foreach ($features as $feature => $info) {
     if ($info['enabled']) {
       $modules[] = $feature;
       // Add recommended modules.
       if (isset($files[$feature]->info['nodestream_recommended'])) {
         foreach($files[$feature]->info['nodestream_recommended'] as $recommended) {
           $modules[] = $recommended;
         }
       }
     }
   }
  }
  // Add NodeStream recommended modules.
  if (isset($files['nodestream']->info['nodestream_recommended'])) {
    foreach ($files['nodestream']->info['nodestream_recommended'] as $recommended) {
      $modules[] = $recommended;
    }
  }

  // Always install required modules first. Respect the dependencies between
  // the modules.
  $required = array();
  $non_required = array();
  // Add modules that other modules depend on.
  foreach ($modules as $module) {
    if ($files[$module]->requires) {
      $modules = array_merge($modules, array_keys($files[$module]->requires));
    }
  }
  $modules = array_unique($modules);
  foreach ($modules as $module) {
    if (!empty($files[$module]->info['required'])) {
      $required[$module] = $files[$module]->sort;
    }
    else {
      $non_required[$module] = $files[$module]->sort;
    }
  }
  arsort($required);
  arsort($non_required);

  $operations = array();
  foreach ($required + $non_required as $module => $weight) {
    $operations[] = array('_install_module_batch', array($module, $files[$module]->info['name']));
  }
  $batch = array(
    'operations' => $operations,
    'title' => st('Installing NodeStream features'),
    'error_message' => st('The installation has encountered an error.'),
    'finished' => '_install_profile_modules_finished',
  );
  return $batch;
}

/**
 * Implements hook_nodestream_feature_types().
 */
function nodestream_nodestream_feature_types() {
  return array(
    'channel' => array(
      'title' => st('Distribution channels'),
      'description' => st('A distribution channel is a way of distributing your content to your users.
        It could be a blog, a web newspaper or a more machine readable channel, like an rss feed.'),
      'collapsed' => FALSE,
    ),
    'content' => array(
      'title' => st('Content'),
      'description' => st('A content feature adds new types of content to your site.
        It could for instance be a blog or an article.'),
    ),
    'extension' => array(
      'title' => st('Extensions'),
      'description' => st('An extension feature adds new functionality to a content feature.
        It could for instance be a fact box for your articles.'),
    ),
  );
}

/**
 * Do things that needs to be done after all modules have been enabled.
 */
function nodestream_finish() {
  // Rebuild default components.
  if (module_exists('defaultconfig')) {
    drupal_flush_all_caches();
    module_list(TRUE);
    defaultconfig_rebuild_all();
  }
}
